# ------------------------------------------------------------------------------
from appy.gen import *

# ------------------------------------------------------------------------------
class Famille:
    '''Famille de formulaires.'''
    description = String(format=String.TEXT)

# ------------------------------------------------------------------------------
class Personne:
    abstract = True
    title = String(show=False)
    prenom = String(multiplicity=(1,1))
    nom = String(multiplicity=(1,1))
    telephone = String()
    courriel = String(validator=String.EMAIL)
    fonction = String()
    fax = String()
    actif = Boolean(default=True)
    def onEdit(self, created):
        self.title = '%s %s' % (self.prenom, self.nom)

class Agent(Personne):
    '''Employe d'une administration ou d'une entreprise.'''
    description = String(show=False)

class AgentInterne(Personne):
    '''Employe de l'administration qui cree les formulaires.'''
    description = String(show=False)

class Ministre(Personne):
    description = String(show=False)

class Employe(Personne):
    '''Employe d'un sous-traitant.'''
    description = String(show=False)

# ------------------------------------------------------------------------------
class Administration:
    '''Administration concernée par la creation d'un formulaire.'''
    gi = 'identification'
    gc = 'coordonnees'
    directionGenerale = String(group=gi)
    division = String(group=gi)
    direction = String(group=gi)
    service = String(group=gi)
    siteWeb = String()
    telephone = String(group=gc)
    fax = String(group=gc)
    emailGeneral = String(group=gc)
    adresse = String(group=gc)
    numero = String(group=gc)
    codePostal = String(group=gc)
    commune = String(group=gc)
    agents = Ref(Agent, multiplicity=(0,None), add=True, link=False,
                 back=Ref(attribute="administration"), showHeaders=True,
                 shownInfo=('fonction','telephone','fax','courriel'))

# ------------------------------------------------------------------------------
class SousTraitant:
    numero = String()
    adresse = String(format=String.TEXT)
    employes = Ref(Employe, multiplicity=(0,None), add=True, link=False,
                   back=Ref(attribute="employeur"), showHeaders=True,
                   shownInfo=('fonction','telephone','fax','courriel'))

# ------------------------------------------------------------------------------
class TypeDeDemarche:
    description = String(format=String.TEXT)

# ------------------------------------------------------------------------------
class ProcessusTravail:
    # Roles
    managerF = 'GestionnaireFormulaires'
    managers = ('Manager', managerF)
    # States
    enAttenteInterne = State({r:managers, w:managers, d:managers}, initial=True)
    enAttenteExterne = State({r:managers, w:managers, d:None})
    enCours = State({r:managers, w:managers, d:None})
    termine = State({r:managers, w:managers, d:None})
    # Transitions
    attribuerEnExterne = Transition( ( (enAttenteInterne, enAttenteExterne),
                                       (enCours, enAttenteExterne)),
                                     condition=managers)
    receptionnerTravailExterne = Transition( (enAttenteExterne, enCours),
                                             condition=managers)
    attribuerEnInterne = Transition( ((enAttenteInterne, enCours),
                                      (enCours, enAttenteInterne)),
                                     condition=managers)
    terminer = Transition( (enCours, termine), condition=managers)
    annuler = Transition( (termine, enCours), condition=managers)

# ------------------------------------------------------------------------------
class Travail:
    '''Represente un type de travail a effectuer sur un formulaire: travail de
       simplification/lisibilite, travail informatique, travail de mise en
       production.'''
    workflow = ProcessusTravail
    title = String(show=False)
    ie = 'IntervenantEtEcheances_3'
    intervenantEasiWal = Ref(AgentInterne, multiplicity=(1,1), add=False,
        link=True, back=Ref(attribute='travauxEnTantQuIntervenantEasiWal'),
        group=ie)
    echeanceDemandee = Date(multiplicity=(1,1), format=Date.WITHOUT_HOUR,
                            group=ie)
    echeancePossible = Date(format=Date.WITHOUT_HOUR, group=ie)
    st = 'Statut_2'
    raisonDeLAttente = String(group=st)
    pourcentageDejaRealise = Integer(group=st)
    dateDeFin = Date(format=Date.WITHOUT_HOUR, group=st)
    dp = 'DocumentsProduits_2'
    documentProduit1 = File(group=dp)
    documentProduit2 = File(group=dp)
    documentProduit3 = File(group=dp)
    documentProduit4 = File(group=dp)
    remarquesEventuelles = String(format=String.XHTML)

    def onEdit(self, created):
        self.title = 'Travail'

    def getStatut(self):
        '''Return status-related info about this work.'''
        res = '<span style="color:red"> - éditez les infos du ' \
              'travail!</span>'
        if not self.echeanceDemandee: return res
        if not self.intervenantEasiWal: return res
        echeance = self.echeanceDemandee.strftime('%d/%m/%Y')
        qui = self.intervenantEasiWal.title.decode('utf-8')
        return '%s (%s, %s)' % (self.stateLabel.decode('utf-8'), echeance, qui)

class TravailInformatique(Travail):
    ie = Travail.ie
    developpeurInterne = Ref(AgentInterne, multiplicity=(0,1), add=False,
        link=True, back=Ref(attribute="travauxEnTantQueDeveloppeur"), group=ie)
    developpeurExterne = Ref(Employe, multiplicity=(0,1), add=False,
        link=True, back=Ref(attribute="travauxEnTantQueDeveloppeur"), group=ie)
    bo = 'ApplicationBackOfficeExistante_2'
    application = String(group=bo)
    descriptionDeLApplication = String(format=String.TEXT, group=bo)
    prestataire = String(group=bo)
    coordonneesDuResponsable = String(group=bo)

# ------------------------------------------------------------------------------
class Formulaire:
    root = True
    eb = {'group': 'EnBref_2'}
    sousTitre = String(**eb)
    natureDeLaDemarche = Ref(TypeDeDemarche, multiplicity=(0,1), add=False,
        link=True, back=Ref(attribute='formulairesRelatifs'), **eb)
    titreAbrege = String(**eb)
    famille = Ref(Famille, multiplicity=(0,1), add=False, link=True,
                  back=Ref(attribute='formulaires'), **eb)
    ac = 'Acteurs'
    numero = Integer(show=False)
    ministreConcerne = Ref(Ministre, multiplicity=(1,1), add=False,
        link=True, group='%s_3' % ac, back=Ref(
        attribute="formulairesCorrespondants"))
    adresseDeRenvoi = Ref(Administration, multiplicity=(1,1), add=False,
        link=True, back=Ref(attribute="formulairesEnTantQuAdresseDeRenvoi"),
        group=ac)
    personnesDeContact = Ref(
        Agent, multiplicity=(1,None), add=False, group=ac, link=True,
        back=Ref(attribute='formulairesEnTantQuePersonneDeContact'))
    publicCible = String(validator=['Citoyen', 'Entreprise', 'Administration',
                                    'Association', 'SyndicatDImmeuble',
                                    'EtablissementScolaire', 'Cpas'], group=ac)
    publicCibleSpecifique = String(group=ac)
    descriptionDetaillee = String(format=String.XHTML)
    oi = {'group': 'AutresInformations_2'}
    descriptionCourte = String(format=String.TEXT, **oi)
    bonASavoir = String(format=String.TEXT, **oi)
    mentions = String(format=String.TEXT, **oi)
    baseLegale = String(format=String.TEXT, **oi)
    delaiDeRenvoiEnJours = Integer(**oi)
    montant = String(**oi)
    frequence = ['faible', 'moyenne', 'forte']
    frequenceDeRemplissageParLeDemandeur=String(validator=frequence, **oi)
    frequenceDeRevisionPotentielleParLAdministration=String(
        validator=frequence, **oi)
    cv = 'ClassementEtVieSurLeSiteOuPortail'
    gcv = {'page': cv, 'group': 'MiseEnLigneEtThemes_2'}
    aMettreEnLigne = Boolean(**gcv)
    misEnLigne = Boolean(**gcv)
    dateDeMiseEnLigne = Date(format=Date.WITHOUT_HOUR, **gcv)
    dateDeFinDeMiseEnLigneEventuelle = Date(format=Date.WITHOUT_HOUR, **gcv)
    themes = ['ActionSocialeEtSante', 'Agriculture',
              'AmenagementDuTerritoireEtUrbanisme', 'Cartographie', 'Emploi',
              'Energie', 'Entreprises', 'EnvironnementEtRessourcesNaturelles',
              'Fiscalite', 'FonctionPublique', 'Formation', 'Logement',
              'MarchesPublics', 'Patrimoine', 'PouvoirsLocaux',
              'RelationsInternationalesEtEurope', 'RechercheEtTechnologies',
              'Ruralite', 'Statistiques', 'Tourisme', 'TransportEtMobilite']
    theme = String(validator=themes, multiplicity=(1, None), **gcv)
    lignesDeVie = ['AvoirDesEnfantsEtEnPrendreSoin',
                   'ApprendreASeFormerToutAuLongDeSaVie',
                   'SeDeplacer', 'SeLoger', 'Travailler',
                   'VieDeFamilleEtViePersonnelle', 'SantePreventionEtSecurite',
                   'ParticiperALaViePubliqueEtEtreEnContactAvecLaJustice',
                   'LoisirsTourismeEtSports', 'ArtEtCulture',
                   'FiscaliteArgentEtConsommation', 'PensionEtSeniors',
                   'PerdreUnProche', 'CreerUneEntreprise', 'GererUneEntreprise',
                   'ArreterUneEntreprise']
    ligneDeVie = String(validator=lignesDeVie, multiplicity=(1, None), **gcv)
    nombreDeFormulairesTelechargesParAn = Integer(**gcv)
    nombreDeFormulairesRemplisEnLigneParAn = Integer(**gcv)
    regles = String(format=String.TEXT, page=cv)
    te = 'TravailAEffectuer'
    teg = {'page': te, 'group': 'InformationsPrincipales_3'}
    origineDeLaDemande = String(multiplicity=(1,1), **teg)
    dateDeReceptionDeLaDemande = Date(format=Date.WITHOUT_HOUR, **teg)
    priorite = String(validator=['Faible', 'Moyenne', 'Forte'], **teg)
    echeanceDemandee = Date(format=Date.WITHOUT_HOUR, **teg)
    echeancePossibleCoteEasi = Date(format=Date.WITHOUT_HOUR, **teg)
    dateDAttributionDuTravail = Date(format=Date.WITHOUT_HOUR, **teg)
    tea = {'page': te, 'group': 'ActeursAdditionnels_3'}
    autresPersonnesDeContact = Ref(
        AgentInterne, multiplicity=(0,None), add=False, link=True,
        back=Ref(attribute='formulairesEnTantQuAutrePersonneDeContact'), **tea)
    fonctionnaireValidant = Ref(Agent, multiplicity=(0,1), add=False,
        link=True, back=Ref(attribute='formulairesEnTantQuAgentValidant'),**tea)
    chefDeProjetCoteEasi = Ref(AgentInterne, multiplicity=(0,1), add=False,
        link=True, back=Ref(attribute='formulairesEnTantQueChefDeProjet'),**tea)
    ted = {'page': te, 'group': 'InformationsRelativesAuDiagnostic_2'}
    dateDeRealisationDuDiagnostic = Date(format=Date.WITHOUT_HOUR, **ted)
    diagnostic = File(**ted)
    tet = {'page': te, 'group': 'TravauxAEffectuer_3', 'multiplicity': (0,1),
           'add': True, 'link': False}
    travailDeSimplification = Ref(Travail, back=Ref(
        attribute='retourAuFormulaireLieACeTravailDeSimplification'), **tet)
    travailDInformatisation = Ref(TravailInformatique, back=Ref(
        attribute='retourAuFormulaireLieACeTravailDInformatisation'), **tet)
    travailDeMiseEnLigne = Ref(Travail, back=Ref(
        attribute='retourAuFormulaireLieACeTravailDeMiseEnLigne'), **tet)
    sy = 'Synthese'
    syu = {'page': sy, 'group': 'gainsUsager'}
    gains = ['clarification', 'blocsCommuns', 'charte', 'nettoyage',
             'confiance', 'collecteUnique', 'notice']
    gainsDef = gains[:4]
    gainsUsagerNiveauSimplification = String(validator=gains, default=gainsDef,
        multiplicity=(0,None), **syu)
    autresGainsUsagerNiveauSimplification = String(format=String.XHTML, **syu)
    gainsInfo = ['encodage', 'disponibilite', 'sauvegarde', 'erreurs', 'envoi',
                 'accuseReception']
    gainsUsagerNiveauInformatique = String(validator=gainsInfo,
        default=gainsInfo, multiplicity=(0,None), **syu)
    autresGainsUsagerNiveauInformatique = String(format=String.XHTML, **syu)
    sya = {'page': sy, 'group': 'gainsAdministration'}
    gainsAdm = ['clarification', 'identification', 'pertinence']
    gainsAdministrationNiveauSimplification = String(validator=gainsAdm,
        default=gainsAdm, multiplicity=(0,None), **sya)
    autresGainsAdministrationNiveauSimplification = String(
        format=String.XHTML, **sya)
    gainsInfoAdm = ['lecture', 'vitesse', 'erreurs', 'envoi']
    gainsAdministrationNiveauInformatique = String(validator=gainsInfoAdm,
        default=gainsInfoAdm, multiplicity=(0,None), **sya)
    autresGainsAdministrationNiveauInformatique = String(
        format=String.XHTML, **sya)
    aa = {'page': sy, 'group': 'EvolutionDuFormulaire_2'}
    nombreDePagesAvant = Integer(**aa)
    nombreDePagesApres = Integer(**aa)
    langues = ['francais', 'allemand', 'neerlandais']
    langueDuFormulaireAvant = String(validator=langues, **aa)
    langueDuFormulaireApres = String(validator=langues, **aa)
    docFormats = ['doc', 'xls', 'xml', 'pdfStatique', 'pdfDynamique']
    formatDeDocumentAvant = String(validator=docFormats, **aa)
    formatDeDocumentApres = String(validator=docFormats, **aa)
    existenceDUneNoticeAvant = Boolean(**aa)
    existenceDUneNoticeApres = Boolean(**aa)
    nombreDePagesDeLaNoticeAvant = Integer(**aa)
    nombreDePagesDeLaNoticeApres = Integer(**aa)
    existenceDAnnexesAvant = Boolean(**aa)
    existenceDAnnexesApres = Boolean(**aa)
    annexesAvant = String(format=String.TEXT, **aa)
    annexesApres = String(format=String.TEXT, **aa)
    nombreDAnnexesQuOnPeutRemplacerParSourcesAuthentiquesAvant=Integer(**aa)
    nombreDAnnexesQuOnPeutRemplacerParSourcesAuthentiquesApres=Integer(**aa)
    nombreDAnnexesRemplaceesParSourcesAuthentiquesAvant=Integer(**aa)
    nombreDAnnexesRemplaceesParSourcesAuthentiquesApres=Integer(**aa)
    annexesRemplaceesAvant = String(format=String.TEXT, **aa)
    annexesRemplaceesApres = String(format=String.TEXT, **aa)
    existenceDAutresDocumentsLiesAvant = Boolean(**aa)
    existenceDAutresDocumentsLiesApres = Boolean(**aa)
    documentsLiesAvant = String(format=String.TEXT, **aa)
    documentsLiesApres = String(format=String.TEXT, **aa)
    typesAvant = ['DocumentPapier', 'DocumentElectronique',
                  'FormulaireInteractif', 'FormulaireIntegre']
    typeDeFormulaireAvant = String(validator=typesAvant, **aa)
    typesApres = ['FormulaireTransactionnelEtIntegre',
                  'FormulaireTransactionnel',
                  'FormulaireInteractif','FormulaireIntegre']
    typeDeFormulaireApres = String(validator=typesApres, **aa)
    respectDeLaCharteAvant = Boolean(**aa)
    respectDeLaCharteApres = Boolean(**aa)
    simplifieAvant = Boolean(**aa)
    simplifieApres = Boolean(**aa)
    simplificationsAvant = String(format=String.TEXT, **aa)
    simplificationsApres = String(format=String.TEXT, **aa)
    diffusionSurSiteFormulairesAvant = Boolean(**aa)
    diffusionSurSiteFormulairesApres = Boolean(**aa)
    autresSitesDeDiffusionAvant = String(format=String.TEXT, **aa)
    autresSitesDeDiffusionApres = String(format=String.TEXT, **aa)
    nombreDeFormulairesDemandesParAnAvant = Integer(**aa)
    nombreDeFormulairesDemandesParAnApres = Integer(**aa)
    # Statut du formulaire = champ derive de l'etat des processus des travaux
    # lies.
    def computeStatus(self):
        '''Computes the status of this form based on sub-works workflow
           states.'''
        res = '<p><b>Simplif</b> ' + self.travailDeSimplification.getStatut()
        if self.travailDInformatisation:
            res += '<br/><b>Info</b> '+self.travailDInformatisation.getStatut()
        if self.travailDeMiseEnLigne:
            res += '<br/><b>Prod</b> ' + self.travailDeMiseEnLigne.getStatut()
        return res+'</p>'
    statut = Computed(method=computeStatus, plainText=False, show=False)

    def onEdit(self, created):
        if created:
            # Update form number.
            n = self.tool.numeroDuDernierFormulaire + 1
            self.numero = n
            self.tool.numeroDuDernierFormulaire = n
        # Create default travail de simplification.
        if not self.travailDeSimplification:
            t = self.create('travailDeSimplification')
# ------------------------------------------------------------------------------
