# ------------------------------------------------------------------------------
from appy.gen import *
from Formulaire import Administration, Ministre, AgentInterne, Famille, \
                       SousTraitant, TypeDeDemarche, Formulaire

# Configuration ----------------------------------------------------------------
c = Config()
c.languages = ('en', 'fr')
c.minimalistPlone = True

# ------------------------------------------------------------------------------
familles = ( ('Formulaires énergie 2007', ''),
             ('Formulaires énergie 2008', '')
)

administrations = ( ('DGTRE', ( ('Christophe', 'Despeghel'), )),
)

agentsInternes = (
    ('Cornet', 'Denis'),
    ('Eloy', 'Marc'),
    ('Jacobs', 'Jean-Christophe'),
    ('Vangeyte', 'Bernard'),
    ('Wanzoul', 'Benoît'),
    ('Weynants', 'Jean-Pierre'),
)

ministres = (
    ('Demotte', 'Rudy'),
    ('Antoine', 'André'),
    ('Daerden', 'Michel'),
    ('Courard', 'Philippe'),
    ('Marcourt', 'Jean-Claude'),
    ('Simonet', 'Marie-Dominique'),
    ('Tarabella', 'Marc'),
    ('Donfut', 'Didier'),
    ('Lutgen', 'Benoît'),
)

sousTraitants = ( ('NSI', ( ('Frédéric', 'Morel'), ) ),
)

typesDeDemarches = (
    ('Prime', ''), ('Subside', ''), ("Rapport d'activité", ''),
    ('Formulaire', ''), ('Agrément', ''), ('Enregistrement', ''),
)

# ------------------------------------------------------------------------------
class ToolOasif(Tool):
    s = {'multiplicity': (0,None), 'add':True, 'link':False, 'showHeaders':True,
         'wide':True}
    back = {'attribute': 'retourALaConfig'}
    numeroDuDernierFormulaire = Integer(default=0)
    familles = Ref(Famille, back=Ref(**back), shownInfo=('description',), **s)
    administrations = Ref(Administration, back=Ref(**back),
                          shownInfo=('directionGenerale', 'division',
                                     'direction', 'service'), **s)
    agentsInternes = Ref(AgentInterne, back=Ref(**back), shownInfo=('actif',),
                         **s)
    ministres = Ref(Ministre, back=Ref(**back), shownInfo=('actif',), **s)
    sousTraitants = Ref(SousTraitant, back=Ref(**back),
                        shownInfo=('numero', 'adresse'), **s)
    typesDeDemarches = Ref(TypeDeDemarche, back=Ref(**back),
                           shownInfo=('description',), **s)
    # Installation
    def onInstall(self):
        if not self.familles:
            # Cree quelques familles de formulaires
            for title, description in familles:
                self.create('familles', title=title, description=description)
            # Cree quelques administrations
            for title, employees in administrations:
                admin = self.create('administrations', title=title)
                for firstName, name in employees:
                    admin.create('agents', prenom=firstName, nom=name)
            # Cree quelques agents internes
            for n, p in agentsInternes:
                self.create('agentsInternes', nom=n, prenom=p)
            # Cree quelques ministres
            for n, p in ministres:
                self.create('ministres', nom=n, prenom=p)
            # Cree quelques sous-traitants
            for title, employees in sousTraitants:
                company = self.create('sousTraitants', title=title)
                for firstName, name in employees:
                    company.create('employes', prenom=firstName, nom=name)
            # Cree les types de démarches
            for t, d in typesDeDemarches:
                self.create('typesDeDemarches', title=t, description=d)
        # Customiser les colonnes d'affichage
        flavour = self.flavours[0]
        flavour.title = 'Tous les formulaires'
        attrName = flavour.getAttributeName('resultColumns', Formulaire)
        columns = ['numero', 'ministreConcerne', 'personnesDeContact', 'statut']
        setattr(flavour, attrName, columns)
        return True
    install = Action(action=onInstall)
# ------------------------------------------------------------------------------
