# -*- coding: utf-8 -*-
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
import Products.Oasif.config
from appy.gen.plone25.mixins.PodTemplateMixin import PodTemplateMixin
from Extensions.appyWrappers import OasifPodTemplate_Wrapper

schema = Schema((
    TextField(
        name='description',
        accessor='Description',
        widget=TextAreaWidget(
            label_msgid='OasifPodTemplate_description',
            i18n_domain='Oasif',
            description='OasifPodTemplate_descriptionDescr',
            description_msgid='OasifPodTemplate_description_descr',
            label='description',
        )
    ),

    FileField(
        required=True,
        storage=AttributeStorage(),
        name='podTemplate',
        widget=FileWidget(
            label_msgid='OasifPodTemplate_podTemplate',
            i18n_domain='Oasif',
            description='OasifPodTemplate_podTemplateDescr',
            description_msgid='OasifPodTemplate_podTemplate_descr',
            label='podTemplate',
        )
    ),

    StringField(
        default='odt',
        enforceVocabulary=True,
        required=True,
        name='podFormat',
        vocabulary='list_podFormat_values',
        widget=SelectionWidget(
            description='OasifPodTemplate_podFormatDescr',
            description_msgid='OasifPodTemplate_podFormat_descr',
            format='select',
            label_msgid='OasifPodTemplate_podFormat',
            label='podFormat',
            i18n_domain='Oasif',
        )
    ),

    StringField(
        default='main',
        name='phase',
        widget=StringWidget(
            description='OasifPodTemplate_phaseDescr',
            description_msgid='OasifPodTemplate_phase_descr',
            label_msgid='OasifPodTemplate_phase',
            label='phase',
            i18n_domain='Oasif',
            size=50,
        )
    ),

),)
fullSchema = BaseSchema.copy() + schema.copy()

class OasifPodTemplate(BaseContent, PodTemplateMixin):
    '''POD template.'''
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),)

    archetype_name = 'OasifPodTemplate'
    meta_type = 'OasifPodTemplate'
    portal_type = 'OasifPodTemplate'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 1
    #content_icon = 'OasifPodTemplate.gif'
    immediate_view = 'Oasif_appy_view'
    default_view = 'Oasif_appy_view'
    suppl_views = ()
    typeDescription = "OasifPodTemplate"
    typeDescMsgId = 'Oasif_edit_descr'
    _at_rename_after_creation = True
    wrapperClass = OasifPodTemplate_Wrapper
    schema = fullSchema

    def at_post_create_script(self): self._appy_onEdit(True)
    def at_post_edit_script(self): self._appy_onEdit(False)
    def post_validate(self, REQUEST=None, errors=None):
        if not errors: self._appy_validateAllFields(REQUEST, errors)
    def getTool(self): return self.portal_oasif
    def getProductConfig(self): return Products.Oasif.config


    def list_podFormat_values(self):
        return self._appy_getDisplayList(('','odt','pdf','rtf','doc',), ('choose_a_value','OasifPodTemplate_podFormat_list_odt','OasifPodTemplate_podFormat_list_pdf','OasifPodTemplate_podFormat_list_rtf','OasifPodTemplate_podFormat_list_doc',), 'Oasif')

registerType(OasifPodTemplate, 'Oasif')
