# -*- coding: utf-8 -*-
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.CMFCore.utils import UniqueObject
import Products.Oasif.config
from appy.gen.plone25.mixins.ToolMixin import ToolMixin
from Extensions.appyWrappers import AbstractWrapper, OasifTool_Wrapper

predefinedSchema = Schema((
    ReferenceField(
        name='flavours',
        relationship='OasifTool_flavours_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_flavoursDescr',
            description_msgid='OasifTool_flavours_descr',
            label_msgid='OasifTool_flavours',
            label='flavours',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    StringField(
        name='unoEnabledPython',
        widget=StringWidget(
            description='OasifTool_unoEnabledPythonDescr',
            description_msgid='OasifTool_unoEnabledPython_descr',
            label_msgid='OasifTool_unoEnabledPython',
            label='unoEnabledPython',
            i18n_domain='Oasif',
            size=50,
        )
    ),

    IntegerField(
        default=2002,
        name='openOfficePort',
        widget=IntegerWidget(
            label_msgid='OasifTool_openOfficePort',
            i18n_domain='Oasif',
            description='OasifTool_openOfficePortDescr',
            description_msgid='OasifTool_openOfficePort_descr',
            label='openOfficePort',
        )
    ),

    IntegerField(
        default=30,
        name='numberOfResultsPerPage',
        widget=IntegerWidget(
            label_msgid='OasifTool_numberOfResultsPerPage',
            i18n_domain='Oasif',
            description='OasifTool_numberOfResultsPerPageDescr',
            description_msgid='OasifTool_numberOfResultsPerPage_descr',
            label='numberOfResultsPerPage',
        )
    ),

    IntegerField(
        default=100,
        name='listBoxesMaximumWidth',
        widget=IntegerWidget(
            label_msgid='OasifTool_listBoxesMaximumWidth',
            i18n_domain='Oasif',
            description='OasifTool_listBoxesMaximumWidthDescr',
            description_msgid='OasifTool_listBoxesMaximumWidth_descr',
            label='listBoxesMaximumWidth',
        )
    ),

),)
schema = Schema((
    IntegerField(
        default=0,
        name='numeroDuDernierFormulaire',
        widget=IntegerWidget(
            label_msgid='OasifTool_numeroDuDernierFormulaire',
            i18n_domain='Oasif',
            description='OasifTool_numeroDuDernierFormulaireDescr',
            description_msgid='OasifTool_numeroDuDernierFormulaire_descr',
            label='numeroDuDernierFormulaire',
        )
    ),

    ReferenceField(
        name='familles',
        relationship='OasifTool_familles_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_famillesDescr',
            description_msgid='OasifTool_familles_descr',
            label_msgid='OasifTool_familles',
            label='familles',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    ReferenceField(
        name='administrations',
        relationship='OasifTool_administrations_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_administrationsDescr',
            description_msgid='OasifTool_administrations_descr',
            label_msgid='OasifTool_administrations',
            label='administrations',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    ReferenceField(
        name='agentsInternes',
        relationship='OasifTool_agentsInternes_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_agentsInternesDescr',
            description_msgid='OasifTool_agentsInternes_descr',
            label_msgid='OasifTool_agentsInternes',
            label='agentsInternes',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    ReferenceField(
        name='ministres',
        relationship='OasifTool_ministres_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_ministresDescr',
            description_msgid='OasifTool_ministres_descr',
            label_msgid='OasifTool_ministres',
            label='ministres',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    ReferenceField(
        name='sousTraitants',
        relationship='OasifTool_sousTraitants_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_sousTraitantsDescr',
            description_msgid='OasifTool_sousTraitants_descr',
            label_msgid='OasifTool_sousTraitants',
            label='sousTraitants',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    ReferenceField(
        name='typesDeDemarches',
        relationship='OasifTool_typesDeDemarches_rel',
        multiValued=True,
        widget=ReferenceWidget(
            description='OasifTool_typesDeDemarchesDescr',
            description_msgid='OasifTool_typesDeDemarches_descr',
            label_msgid='OasifTool_typesDeDemarches',
            label='typesDeDemarches',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

    StringField(
        name='install',
        widget=StringWidget(
            description='OasifTool_installDescr',
            description_msgid='OasifTool_install_descr',
            label_msgid='OasifTool_install',
            label='install',
            visible=False,
            i18n_domain='Oasif',
        )
    ),

),)
fullSchema = OrderedBaseFolderSchema.copy() + predefinedSchema.copy() + schema.copy()

class OasifTool(UniqueObject, OrderedBaseFolder, ToolMixin):
    '''Tool for Oasif.'''
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(OrderedBaseFolder,'__implements__',()),)

    archetype_name = 'OasifTool'
    meta_type = 'OasifTool'
    portal_type = 'OasifTool'
    allowed_content_types = ()
    filter_content_types = 0
    global_allow = 0
    #content_icon = 'OasifTool.gif'
    immediate_view = 'Oasif_appy_view'
    default_view = 'Oasif_appy_view'
    suppl_views = ()
    typeDescription = "OasifTool"
    typeDescMsgId = 'OasifTool_edit_descr'
    i18nDomain = 'Oasif'
    wrapperClass = OasifTool_Wrapper
    _at_rename_after_creation = True
    schema = fullSchema
    schema["id"].widget.visible = False
    schema["title"].widget.visible = False
    # When browsing into the tool, the 'configure' portlet should be dislayed.
    left_slots = ['here/portlet_prefs/macros/portlet']
    right_slots = []

    # Tool constructor has no id argument, the id is fixed.
    def __init__(self, id=None):
        OrderedBaseFolder.__init__(self, 'portal_oasif')
        self.setTitle('Oasif')

    def at_post_create_script(self): self._appy_onEdit(True)
    def at_post_edit_script(self): self._appy_onEdit(False)
    def post_validate(self, REQUEST=None, errors=None):
        if not errors: self._appy_validateAllFields(REQUEST, errors)
    def getTool(self): return self.portal_oasif
    def getProductConfig(self): return Products.Oasif.config


    def validate_flavours(self, value):
        return self._appy_validateField('flavours', value, 'OasifTool_flavours_valid', 'Ref')

    def validate_openOfficePort(self, value):
        return self._appy_validateField('openOfficePort', value, 'OasifTool_openOfficePort_valid', 'Integer')

    def validate_numberOfResultsPerPage(self, value):
        return self._appy_validateField('numberOfResultsPerPage', value, 'OasifTool_numberOfResultsPerPage_valid', 'Integer')

    def validate_listBoxesMaximumWidth(self, value):
        return self._appy_validateField('listBoxesMaximumWidth', value, 'OasifTool_listBoxesMaximumWidth_valid', 'Integer')

    def validate_unoEnabledPython(self, value):
        return self._appy_validateUnoEnabledPython(value)


    def validate_numeroDuDernierFormulaire(self, value):
        return self._appy_validateField('numeroDuDernierFormulaire', value, 'OasifTool_numeroDuDernierFormulaire_valid', 'Integer')

    def validate_familles(self, value):
        return self._appy_validateField('familles', value, 'OasifTool_familles_valid', 'Ref')

    def validate_administrations(self, value):
        return self._appy_validateField('administrations', value, 'OasifTool_administrations_valid', 'Ref')

    def validate_agentsInternes(self, value):
        return self._appy_validateField('agentsInternes', value, 'OasifTool_agentsInternes_valid', 'Ref')

    def validate_ministres(self, value):
        return self._appy_validateField('ministres', value, 'OasifTool_ministres_valid', 'Ref')

    def validate_sousTraitants(self, value):
        return self._appy_validateField('sousTraitants', value, 'OasifTool_sousTraitants_valid', 'Ref')

    def validate_typesDeDemarches(self, value):
        return self._appy_validateField('typesDeDemarches', value, 'OasifTool_typesDeDemarches_valid', 'Ref')

registerType(OasifTool, 'Oasif')
